@@ -23,7 +23,7 @@
  *  doi:10.1109/TKDE.2016.2551240
  *
  *
- *  Copyleft (C) 2018-2020, Marek Gagolewski <https://www.gagolewski.com>
+ *  Copyleft (C) 2018-2022, Marek Gagolewski <https://www.gagolewski.com>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Affero General Public License
@@ -101,39 +101,129 @@ struct CComparePartitionsInfoResult {
 
 
 
+/*!
+ * Stores normalised set-matching scores
+ */
+struct CCompareSetMatchingResult {
+    double psi;
+    double spsi;
+};
 
 
-/*! Applies partial pivoting to a given confusion matrix - permutes the columns
- *  so as to have the largest elements in each row on the main diagonal.
+
+
+/*! Normalising permutation for the columns of a confusion matrix
  *
- *  This comes in handy whenever C actually summarises the results generated
+ *  Determines the reordering of columns in a given confusion matrix
+ *  so that the sum of the elements on the main diagonal is the largest
+ *  possible (by solving the maximal assignment problem).
+ *
+ *  Comes in handy if C summarises the results generated
  *  by clustering algorithms, where actual label values do not matter
  *  (e.g., (1, 2, 0) can be remapped to (0, 2, 1) with no change in meaning.
  *
  *
- * @param C [in/out] a c_contiguous confusion matrix of size xc*yc
- * @param xc number of rows in C
- * @param yc number of columns in C
+ *  @param C a c_contiguous confusion matrix of size xc*yc
+ *  @param xc number of rows in C
+ *  @param yc number of columns in C; xc <= yc
+ *  @param Iout [out] output sequence of length yc
  *
- * Note that C is modified in-place (overwritten).
+ *  Note that Iout is modified in-place (overwritten).
  */
-template<class T>
-void Capply_pivoting(T* C, ssize_t xc, ssize_t yc)
-{
-    for (ssize_t i=0; i<std::min(xc-1, yc-1); ++i) {
-        ssize_t w = i;
-        for (ssize_t j=i+1; j<yc; ++j) {
-            // find w = argmax C[i,w], w=i,i+1,...yc-1
-            if (C[i*yc+w] < C[i*yc+j]) w = j;
-        }
-        for (ssize_t j=0; j<xc; ++j) {
-            // swap columns i and w
-            std::swap(C[j*yc+i], C[j*yc+w]);
+template<class T1, class T2>
+void Cnormalizing_permutation(
+    const T1* C, ssize_t xc, ssize_t yc, T2* Iout
+) {
+    GENIECLUST_ASSERT(xc <= yc);
+
+    std::vector<bool> column_used(yc, false);
+
+    ssize_t retval = linear_sum_assignment(
+        C, xc, yc, Iout, /*minimise*/false
+    );
+    GENIECLUST_ASSERT(retval == 0);
+
+    // only Iout[0]..Iout[xc-1] are set
+    ssize_t i;
+    for (i=0; i<xc; ++i) {
+        column_used[ Iout[i] ] = true;
+    }
+
+    // the remainder:
+    for (ssize_t k=0; k<yc; ++k) {
+        if (!column_used[k]) {
+            column_used[k] = true;
+            Iout[i] = k;
+            i++;
+
+            if (i == yc) break;
         }
     }
 }
 
 
+/*! Applies pivoting to a given confusion matrix
+ *
+ *  Permutes the rows and columns so that the sum of the elements
+ *  on the main diagonal is the largest possible (by solving
+ *  the maximal assignment problem).
+ *
+ *  See Cnormalizing_permutation().
+ *
+ *
+ *  @param C a c_contiguous confusion matrix of size xc*yc
+ *  @param xc number of rows in C
+ *  @param yc number of columns in C; xc <= yc
+ *  @param Cout [out] output matrix after pivoting
+ *
+ *  Note that Cout is modified in-place (overwritten).
+ */
+template<class T>
+void Capply_pivoting(
+    const T* C, ssize_t xc, ssize_t yc, T* Cout/*, bool use_sum=false*/
+) {
+    GENIECLUST_ASSERT(xc <= yc);
+
+//     if (use_sum) {
+
+    std::vector<ssize_t> output_col4row(yc);
+
+    Cnormalizing_permutation(C, xc, yc, /*retval*/output_col4row.data());
+
+    ssize_t i;
+    for (i=0; i<yc; ++i) {
+        for (ssize_t j=0; j<xc; ++j)
+            Cout[yc*j+i] = C[yc*j+output_col4row[i]];
+    }
+
+//     }
+//     WARNING: not tested yet
+//     else { // use max
+//         for (ssize_t ij=0; ij<xc*yc; ++ij)
+//             Cout[ij] = C[ij];
+//
+//         for (ssize_t i=0; i<xc-1; ++i) {
+//             ssize_t wi = i, wj = i;
+//
+//             for (ssize_t ni=i; ni<xc; ++ni) {
+//                 for (ssize_t nj=i; nj<yc; ++nj) {
+//                 // find wi, wj = argmax C[ni,nj]
+//                 if (C[wi*yc+wj] < C[ni*yc+nj]) { wi = ni; wj = nj; }
+//             }
+//             // swap columns i and wj
+//             for (ssize_t j=0; j<xc; ++j) {
+//                 std::swap(C[j*yc+i], C[j*yc+wj]);
+//             }
+//             // swap rows i and wi
+//             for (ssize_t k=0; k<yc; ++k) {
+//                 std::swap(C[i*yc+k], C[wi*yc+k]);
+//             }
+//         }
+//     }
+}
+
+
+
 /*! Computes the confusion matrix (as a dense matrix) - a 2-way contingency table
  *
  * @param C [out] a c_contiguous matrix of size xc*yc
@@ -150,17 +240,17 @@ void Capply_pivoting(T* C, ssize_t xc, ssize_t yc)
  * The elements in C are modified in-place.
  */
 template<class T>
-void Ccontingency_table(T* C, ssize_t xc, ssize_t yc,
+void Ccontingency_table(T* Cout, ssize_t xc, ssize_t yc,
         T xmin, T ymin,
         const T* x, const T* y, ssize_t n)
 {
     for (ssize_t j=0; j<xc*yc; ++j)
-        C[j] = 0;
+        Cout[j] = 0;
 
     for (ssize_t i=0; i<n; ++i) {
         GENIECLUST_ASSERT(   0 <= (x[i]-xmin)*yc +(y[i]-ymin));
         GENIECLUST_ASSERT(xc*yc > (x[i]-xmin)*yc +(y[i]-ymin));
-        C[(x[i]-xmin)*yc +(y[i]-ymin)]++;
+        Cout[(x[i]-xmin)*yc +(y[i]-ymin)]++;
     }
 }
 
@@ -313,22 +403,25 @@ CComparePartitionsInfoResult Ccompare_partitions_info(const T* C,
 
 /*! Computes the normalised accuracy score between two partitions
  *
- *  Normalised accuracy is (Accuracy(C[sigma])-1.0/yc)/(1.0-1.0/yc),
- *  where C[sigma] is a version of the input confusion matrix
+ *  Normalised accuracy is (Accuracy(C[:,sigma])-1.0/yc)/(1.0-1.0/yc),
+ *  where C[:,sigma] is a version of the input confusion matrix
  *  with columns permuted based on the solution to the
  *  maximal linear sum assignment problem.
  *
- *  Accuracy(C[sigma]) is sometimes referred to as purity,
- *  e.g. in (Rendon et al. 2011).
+ *  Accuracy(C[:,sigma]) is sometimes referred to as
+ *  set-matching classification rate or pivoted accuracy.
  *
  *
  *  References
  *  ==========
  *
- *  Rendon E., Abundez I., Arizmendi A., Quiroz E.M.,
- *  Internal versus external cluster validation indexes,
- *  International Journal of Computers and Communications 5(1), 2011, pp. 27-34.
-
+ *  Steinley D., Properties of the Hubert-Arabie adjusted Rand index,
+ *  Psychological Methods 9(3), 2004, pp. 386-396,
+ *  doi:10.1037/1082-989X.9.3.386.
+ *
+ *  Meila M., Heckerman D., An experimental comparison of model-based clustering
+ *  methods, Machine Learning 42, 2001, pp. 9--29, doi:10.1023/A:1007648401407.
+ *
  *
  *  @param C a c_contiguous confusion matrix of size xc*yc
  *  @param xc number of rows in C, xc <= yc
@@ -354,16 +447,68 @@ double Ccompare_partitions_nacc(const T* C, ssize_t xc, ssize_t yc)
     for (ssize_t i=0; i<xc; ++i)
         t += C[yc*i+output_col4row[i]];
 
-    double pur = (double)t/(double)n;
-    return (pur-1.0/yc)/(1.0-1.0/yc);
+    double a = (double)t/(double)n;
+    return (a*yc-1.0)/(yc-1.0);
 
 }
 
 
 
+/*! Computes the adjusted asymmetric accuracy (AAA) score
+ *
+ *  AAA is asymmetric - we assume that rows in C determine the true (reference)
+ *  partition.
+ *
+ *  References
+ *  ==========
+ *
+ *  Gagolewski M., Adjusted asymmetric accuracy: A well-behaving external
+ *  cluster validity measure, 2022, submitted for publication.
+ *
+ *
+ *  @param C a c_contiguous confusion matrix of size xc*yc
+ *  @param xc number of rows in C, xc == yc
+ *  @param yc number of columns in C
+ *
+ *  @return the computed score
+ */
+template<class T>
+double Ccompare_partitions_aaa(const T* C, ssize_t xc, ssize_t yc)
+{
+    GENIECLUST_ASSERT(xc == yc);
+
+    std::vector<double> sum_x(xc);
+    for (ssize_t i=0; i<xc; ++i) {
+        for (ssize_t j=0; j<yc; ++j) {
+            sum_x[i] += C[i*yc+j];
+        }
+    }
+
+    std::vector<double> S(xc*yc);
+    for (ssize_t i=0; i<xc; ++i) {
+        for (ssize_t j=0; j<yc; ++j) {
+            S[i*yc+j] = (double)C[i*yc+j]/(double)sum_x[i];
+        }
+    }
+
+    std::vector<ssize_t> output_col4row2(xc);
+    ssize_t retval = linear_sum_assignment(S.data(), xc, yc, output_col4row2.data(), false); // minimise=false
+    GENIECLUST_ASSERT(retval == 0);
+
+    double t = 0.0;
+    for (ssize_t i=0; i<xc; ++i)
+        t += S[yc*i+output_col4row2[i]];
+
+    return (t-1.0)/(yc-1.0);
+
+}
 
 
-/*! Computes the PSI (pair sets index) score
+/*! Computes the pair sets index (PSI) and its simplified version
+ *
+ *
+ *  SPSI (simplified PSI) assumes E=1 in the definition of the index
+ *  in (Rezaei, Franti 2016), i.e., uses Eq. (20) instead of (18) therein.
  *
  *  References
  *  ==========
@@ -372,15 +517,17 @@ double Ccompare_partitions_nacc(const T* C, ssize_t xc, ssize_t yc)
  *  IEEE Transactions on Knowledge and Data Mining 28(8), 2016, pp. 2173-2186,
  *  doi:10.1109/TKDE.2016.2551240
  *
+ *
  *  @param C a c_contiguous confusion matrix of size xc*yc
  *  @param xc number of rows in C, xc <= yc
  *  @param yc number of columns in C
  *
- *  @return the computed score
+ *  @return the computed scores
  */
 template<class T>
-double Ccompare_partitions_psi(const T* C, ssize_t xc, ssize_t yc)
-{
+CCompareSetMatchingResult Ccompare_partitions_psi(
+    const T* C, ssize_t xc, ssize_t yc
+) {
     GENIECLUST_ASSERT(xc <= yc);
 
     double n = 0.0; // total sum (length of the underlying x and y = number of points)
@@ -410,17 +557,23 @@ double Ccompare_partitions_psi(const T* C, ssize_t xc, ssize_t yc)
     for (ssize_t i=0; i<xc; ++i)
         s += S[yc*i+output_col4row2[i]];
 
+    double es;
     std::sort(sum_x.begin(), sum_x.end());
     std::sort(sum_y.begin(), sum_y.end());
-    double es = 0.0;
+    es = 0.0;
     for (ssize_t i=0; i<xc; ++i)
         es += sum_y[yc-i-1]*sum_x[xc-i-1]/(double)std::max(sum_x[xc-i-1], sum_y[yc-i-1]);
     es /= (double)n;
 
-    double psi  = (s-es)/(yc-es);
-    if (psi<0.0) psi = 0.0;
+    CCompareSetMatchingResult res;
 
-    return psi;
+    res.psi  = (s-es)/(yc-es);
+    if (res.psi<0.0) res.psi = 0.0;
+
+    res.spsi  = (s-1.0)/(yc-1.0);
+    if (res.spsi<0.0) res.spsi = 0.0;
+
+    return res;
 }
 
 
